// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/user.dart';
import 'package:meta/meta.dart';

const String _messageEventType = 'm.room.message';

abstract class MessageEventContent extends EventContent {

  @override
  final String matrixType = _messageEventType;

  @protected
  String get messageType;

  @override
  Map<String, dynamic> toJson() => {
    'msgtype': messageType
  };
}

abstract class MessageEvent extends RoomEvent {

  static const String type = _messageEventType;

  @override
  final String matrixType = _messageEventType;

  @override
  final MessageEventContent content;

  MessageEvent(
    this.content,
    EventId id,
    Room room,
    User sender,
    DateTime time,
    {SentState sentState = SentState.received})
    : super(id, room, sender, time, sentState: sentState);

  factory MessageEvent.fromJson(
      EventId id, Room room, User sender,
      DateTime time, Map<String, dynamic> json) {

    var msgtype = json['content']['msgtype'];

    switch (msgtype) {
      case TextMessage.type:
      default:
        final body = json['content']['body'];
        return TextMessageEvent(
          TextMessage(body: body), id, room, sender, time);
    }
  }
}

class TextMessage extends MessageEventContent {
  static const String type = 'm.text';
  @override
  final String messageType = type;

  final String body;

  TextMessage({@required this.body});

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'body': body
    });
}

class TextMessageEvent extends MessageEvent {

  @override
  final TextMessage content;

  TextMessageEvent(
    this.content,
    EventId id,
    Room room,
    User sender,
    DateTime time,
    {SentState sentState = SentState.received})
    : super(content, id, room, sender, time, sentState: sentState);
}